/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;

/**
 * This class is a simple container for services (by default request URIs).  
 * The monitor simply contains information about whether or not the given
 * service is active or not, and if it is not active, when it wil be 
 * reactivated.  
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 */
public class AppSensorServiceMonitor {
	
	/** The service considered for this service monitor, set in constructor */
	private final String service;
	
	/** The user id associated with this service - if there is one */
    private final String userId;
	
	/** whether or not the service is active, defaults to active */
	private boolean active = true;
	
	/** time to reactivate */
	private long reActivateTime = 0;
	
	/** number used to represent a permanent lock */
	public final static int PERMANENT_LOCK_DURATION = -1;
	
	/** Constant for second time duration */
	public final static String SECOND = "s";
	
	/** Constant for minute time duration */
	public final static String MINUTE = "m";
	
	/** Constant for hour time duration */
	public final static String HOUR = "h";
	
	/** Constant for day time duration */
	public final static String DAY = "d";
	
	/** time scales that are supported (second, minute, hour, day) */
	public static Collection<String> acceptableTimeScales = new ArrayList<String>();
	
	static {
		acceptableTimeScales.add(SECOND);
		acceptableTimeScales.add(MINUTE);
		acceptableTimeScales.add(HOUR);
		acceptableTimeScales.add(DAY);
	}
	
	/** Create a new monitor for the specified service */
	public AppSensorServiceMonitor(String service, final String userId) {
		this.service = service;
		this.userId = userId;
	}

	/**
	 * This method disables the service represented by this monitor
	 * for the specified duration and time scale.  For instance
	 * if the duration is "30", and the timescale is "m", the current
	 * service would be disabled for 30 minutes.
	 * 
	 * @param duration represents unit of measure for how long 
	 * 			to disable the current service, combined with timeScale
	 * @param timeScale represents the scale of time to disable the 
	 * 			current service, combined with duration 
	 */
	public void disable(int duration, String timeScale) {
		Calendar cal = DateUtils.getCurrentCalendar();
		
		if (! acceptableTimeScales.contains(timeScale)) {
			//if we got here, it means someone has requested an unsupported timeScale
			throw new IllegalArgumentException("There has been a request for an timeScale " +
					"that is not supported by this service monitor.  The requested timeScale is: " + timeScale);
		}

		if (duration < 0) {
			//negative duration
			throw new IllegalArgumentException("A negative duration is not an acceptable timespan.");
		}
		
		//handle indefinite request first 
		if (duration == PERMANENT_LOCK_DURATION) {
			cal.add(Calendar.YEAR, 1000);	//adding 1,000 years - same as indefinite
		} else {
			//handle adding duration based on requested scale type
			
			if (SECOND.equalsIgnoreCase(timeScale)) {
				cal.add(Calendar.SECOND, duration);
			}
			
			if (MINUTE.equalsIgnoreCase(timeScale)) {
				cal.add(Calendar.MINUTE, duration);
			}
			
			if (HOUR.equalsIgnoreCase(timeScale)) {
				cal.add(Calendar.HOUR, duration);
			}
			
			if (DAY.equalsIgnoreCase(timeScale)) {
				cal.add(Calendar.DATE, duration);
			}
		}
		
		//go ahead and set reactivate time after doing calendar manipulation
		this.reActivateTime = cal.getTimeInMillis();
		
		//set this service to disabled
		this.active = false;
	}

	/**
	 * Determine whether or not this service is active
	 * @return true if service is active, false if not
	 */
	public boolean isActive() {
		return this.active;
	}

	/**
	 * Get time service will be re-activated
	 * @return time service will be re-activated
	 */
	public long getReActivateTime() {
		return this.reActivateTime;
	}

	/**
	 * Enable this service
	 */
	public void enable() {
		this.active = true;
		this.reActivateTime = 0;
	}
	
	/**
	 * Get the name of the service being monitored
	 * @return the name of the service being monitored
	 */
	public String getService() {
		return this.service;
	}
	
	/**
     * Get the user id for whom this service belongs (may be null)
     * @return user id
     */
    public String getUserId() {
        return this.userId;
    }
}
