package org.owasp.appsensor.filters;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.owasp.appsensor.APPSENSOR;
import org.owasp.appsensor.AppSensorServiceController;
import org.owasp.esapi.StringUtilities;

/**
 * This class is a servlet filter that can be configured in the web.xml to 
 * provide default handling of blocking requests that are managed by 
 * AppSensor.  
 * The configuration in web.xml should be setup like this.
 * 
 * <filter>
 *    	<filter-name>AppSensorRequestBlockingFilter</filter-name>
 *    	<filter-class>
 *			org.owasp.appsensor.filters.AppSensorRequestBlockingFilter
 *		</filter-class>
 *    	<init-param>
 *     		<param-name>redirectURL</param-name>
 *     		<param-value>/errors/requestBlocked.jsp</param-value>
 *    	</init-param>
 * 	</filter>
 * 	<filter-mapping>
 *	    <filter-name>AppSensorRequestBlockingFilter</filter-name>
 *	    <url-pattern>/*</url-pattern>
 * 	</filter-mapping>
 * 
 * The "time" will be sent as a long to your configured page, and you can
 * then display this as you see fit
 */
public class AppSensorRequestBlockingFilter implements Filter {

	private String redirectURL = "appsensor_locked.jsp";
	
	/**
	 * Default constructor.
	 */
	public AppSensorRequestBlockingFilter() {
	}

	/**
	 * @see Filter#destroy()
	 */
	public void destroy() {
	}

	/**
	 * @see Filter#doFilter(ServletRequest, ServletResponse, FilterChain)
	 */
	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
		HttpServletRequest request = (HttpServletRequest) req;
		HttpServletResponse response = (HttpServletResponse) res;
	
		String requestURI = request.getRequestURI();
		
		//set on thread local so it's accessible later
		AppSensorServiceController.setCurrentRequestURI(requestURI);
		
		// is page disabled or active?
		boolean isActive = AppSensorServiceController.isServiceActive(requestURI);
		if (!(isActive)) {
			Long reactivation = AppSensorServiceController.getServiceReactivationTime(requestURI, APPSENSOR.asUtilities().getCurrentUser());
			response.sendRedirect(redirectURL + "?time=" + reactivation);
			// important: can't add chain.dofilter here. else, blocking by appsensor is negated
		} else { 
			// send the user to the page
			chain.doFilter(req, res);
		}
	}

	/**
	 * @see Filter#init(FilterConfig)
	 */
	public void init(FilterConfig filterConfig) throws ServletException {
		this.redirectURL = StringUtilities.replaceNull( filterConfig.getInitParameter( "redirectURL" ), redirectURL );
	}

}
