package org.owasp.appsensor.filters;

import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.owasp.appsensor.errors.AppSensorException;

/**
 * This class is a servlet filter that can be configured in the web.xml to 
 * detect if the IP address associated with a given session id changes mid-session.  
 * This could point to an attacker trying to reuse a session id from another machine.
 * 
 * The configuration in web.xml should be setup like this.
 * 
 * <filter>
 *    	<filter-name>IpAddressChangeDetectionFilter</filter-name>
 *    	<filter-class>
 *			org.owasp.appsensor.filters.IpAddressChangeDetectionFilter
 *		</filter-class>
 * 	</filter>
 * 	<filter-mapping>
 *	    <filter-name>IpAddressChangeDetectionFilter</filter-name>
 *	    <url-pattern>/*</url-pattern>
 * 	</filter-mapping>
 * 
 */
public class IpAddressChangeDetectionFilter implements Filter {

	/** Map that holds mappings of session id's to client IP addresses. */
	//jtm - 5/23/2011 - change to concurrent map to address http://code.google.com/p/appsensor/issues/detail?id=6
	private static ConcurrentMap<String,String> sessionClientIPMap = new ConcurrentHashMap<String,String>();
	
	/**
	 * Default constructor.
	 */
	public IpAddressChangeDetectionFilter() {
	}

	/**
	 * @see Filter#destroy()
	 */
	public void destroy() {
	}

	/**
	 * @see Filter#doFilter(ServletRequest, ServletResponse, FilterChain)
	 */
	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
		HttpServletRequest request = (HttpServletRequest) req;
		
		HttpSession session = request.getSession(false);
		String clientIP = request.getRemoteAddr();
		
		//check if we can proceed
		if (session != null && session.getId() != null && clientIP != null) {
			String sessionId = session.getId();
			
			//if session id is new to tracking - just add it to map
			if (! sessionClientIPMap.containsKey(sessionId)) {
				//http://dmy999.com/article/34/correct-use-of-concurrenthashmap
				sessionClientIPMap.putIfAbsent(sessionId, clientIP);
			} else {	
				//session id is already in map - check if clientIP matches
				String existingClientIP = sessionClientIPMap.get(sessionId);
				
				//if there is a discrepancy, report the exception
				if (! clientIP.equals(existingClientIP)) {
					new AppSensorException("SE5", "AppSensorUser Message SE5", 
							"Client IP Address for session has changed - original client IP [" + existingClientIP + "]" +
							" / new client IP [" + clientIP + "] for session id [" + sessionId + "]");
				}
			}
			
		}
		
		//forward on the user
		chain.doFilter(req, res);
	}

	/**
	 * @see Filter#init(FilterConfig)
	 */
	public void init(FilterConfig filterConfig) throws ServletException {
	}

}
