/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection.reference;

import java.util.List;
import java.util.Properties;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.owasp.appsensor.APPSENSOR;
import org.owasp.appsensor.ASLogger;
import org.owasp.appsensor.ASUser;
import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.AppSensorSecurityConfiguration;
import org.owasp.appsensor.AppSensorServiceController;
import org.owasp.appsensor.AppSensorThreshold;
import org.owasp.appsensor.intrusiondetection.AppSensorIntrusionDetector;
import org.owasp.appsensor.intrusiondetection.ResponseAction;

/**
 * Reference implementation of the ResponseAction interface. This 
 * implementation handles the following response actions: 
 * 
 * <ul>
 * 		<li>"log" - logs the activity</li>
 *  	<li>"logout" - logs the currently logged in user out (if one exists)</li>
 *   	<li>"disable" - disables the account of the currently logged in user (if one exists)</li>
 *   	<li>"disableComponent" - disables access to the location of the intrusion using 
 *   		the AppSensorServiceController
 *   	</li> 
 *   	<li>"disableComponentForUser" - disables access to the location of the intrusion using 
 *   		the AppSensorServiceController for the currently logged in user(if one exists)
 *   	</li>    
 *   	<li>"smsAdmin" - sends sms to Administrator whose number is configured in appsensor.properties file
 *   	</li>   
 *   	<li>"emailAdmin" - sends email to Administrator whose address is configured in appsensor.properties file
 *   	</li>    
 * </ul>
 * 
 * The configuration point to specify another class that implements the ResponseAction interface
 * is in the appsensor.properties file in the .esapi directory.
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 * @see org.owasp.appsensor.intrusiondetection.ResponseAction
 */
public class DefaultResponseAction implements ResponseAction {
	
	private static volatile ResponseAction singletonInstance;

    public static ResponseAction getInstance()
    {
        if ( singletonInstance == null ) {
            synchronized ( DefaultResponseAction.class ) {
                if ( singletonInstance == null ) {
                    singletonInstance = new DefaultResponseAction();
                }
            }
        }
        return singletonInstance;
    }
	
	private final static ASLogger logger = APPSENSOR.asUtilities().getLogger("DefaultResponseAction"); 

	private enum MailType { EMAIL, SMS }
	private static int SMS_MESSAGE_LENGTH = 160;
	
	/**
	 * {@inheritDoc}
	 */
	public boolean handleResponse(String action, AppSensorIntrusion currentIntrusion) {

		if (action != null) {
			//log issue
			if (action.equalsIgnoreCase("log")) {
				return logEvent(currentIntrusion);
			}
			
			//disable access to this component for all users
			if (action.equalsIgnoreCase("disableComponent")) {
				return disableComponent(currentIntrusion);
			}
			
			//sms administrator (via email to sms account) to notify of what action has occurred
			if (action.equalsIgnoreCase("smsAdmin")) {
				return smsAdmin(currentIntrusion);
			}
			
			//email administrator to notify of what action has occurred
			if (action.equalsIgnoreCase("emailAdmin")) {
				return emailAdmin(currentIntrusion);
			}
			
			//can't perform the following actions if the user isn't logged in
			ASUser user = APPSENSOR.asUtilities().getCurrentUser();
	        if (user.isAnonymous()) {
	        	logger.debug("Can't process this action as there is an anonymous user.");
	        	return true;
	        }
	        
	        //disable the user
			if (action.equalsIgnoreCase("disable")) {
				return disable(currentIntrusion.getUser());
			}
			
			//log the user out
			if (action.equalsIgnoreCase("logout")) {
				return logout(currentIntrusion.getUser());
			}
			
			//disable access to this component for a specific user
			if (action.equalsIgnoreCase("disableComponentForUser")) {
				return disableComponentForUser(currentIntrusion);
			}
		}
		
		//if we got here, it means someone has requested an unsupported action
		throw new IllegalArgumentException("There has been a request for an action " +
				"that is not supported by this response handler.  The requested action is: " + action);
	}
	
	/**
	 * Log the current event
	 * @param user currently logged in user
	 * @param currentIntrusion current intrusion 
	 * @return true if successful
	 */
	private boolean logEvent(AppSensorIntrusion currentIntrusion) {
		ASUser user = currentIntrusion.getUser();
		logger.debug("Response Action: Logging issue performed by " + user.getAccountId());
		logger.fatal("INTRUSION - Multiple intrusions observed by AppSensorUser:" + user.getAccountName() + 
				", UserID:" + user.getAccountId() + ", Action: Logging Event with code: " + currentIntrusion.getEventCode());
		return true;
	}

	/**
	 * Logout the currently logged in user
	 * @param user currently logged in user
	 * @return true if successful
	 */
	private boolean logout(ASUser user) {
		logger.debug("Response Action: Logging Out User " + user.getAccountId());
		logger.fatal("INTRUSION - Multiple intrusions observed by AppSensorUser:" + 
				user.getAccountName() + ", UserID:" + user.getAccountId() + ", Action: Logging out malicious account");
		user.logout();
		return true;
	}

	/**
	 * Disable the currently logged in user
	 * @param user currently logged in user
	 * @return true if successful
	 */
	private boolean disable(ASUser user) {
		logger.debug("Response Action: Disabling User " + user.getAccountId());
		logger.fatal("INTRUSION - Multiple intrusions observed by AppSensorUser:" + 
				user.getAccountName() + ", UserID:" + user.getAccountId() + ", Action: Disabling and logging out malicious account");
		user.disable();
		user.logout();
		return true;
	}
	
	/**
	 * Disables the function associated with the current intrusion for the 
	 * designated period of time.  
	 * @param currentIntrusion current intrusion
	 * @return true if successful
	 */
	private boolean disableComponent(AppSensorIntrusion currentIntrusion) {
		//retrieve threshold from config file
		AppSensorThreshold threshold = AppSensorIntrusionDetector.getAppSensorThreshold(currentIntrusion.getEventCode());
		
		//determine service to disable based on current intrusion
		String serviceToDisable = currentIntrusion.getLocation();
		
		if (threshold != null) {
			//get amount of time to disable component for
			int duration = threshold.disableComponentDuration;
			String timeScale = threshold.disableComponentTimeScale;
			
			//actually disable service for all users
			AppSensorServiceController.disableService(serviceToDisable, duration, timeScale);
			logger.warning("Successfully disabled service: Suspending Service <" + serviceToDisable +
					"> for <" + duration + "> <" + timeScale + ">");
			return true;
		} else {
			logger.warning("Could not disabled service <" + serviceToDisable + "> because" +
					" there is no configuration setup in the ESAPI.properties file");
			return false;
		}
	}
	
	/**
	 * Disables the function associated with the current intrusion for the 
	 * designated period of time, but only for the specified user.
	 * @param currentIntrusion current intrusion
	 * @return true if successful
	 */
	private boolean disableComponentForUser(AppSensorIntrusion currentIntrusion) {
		//retrieve threshold from config file
		AppSensorThreshold threshold = AppSensorIntrusionDetector.getAppSensorThreshold(currentIntrusion.getEventCode());
		
		//determine service to disable based on current intrusion
		String serviceToDisable = currentIntrusion.getLocation();
		
		//retrieve user from intrusion
		ASUser user = currentIntrusion.getUser();
		//get userid of user
		String userId = String.valueOf(user.getAccountId());
		
		if (threshold != null) {
			//get amount of time to disable component for
			int duration = threshold.disableComponentForUserDuration;
			String timeScale = threshold.disableComponentForUserTimeScale;
		
			//actually disable service for user
			AppSensorServiceController.disableServiceForUser(serviceToDisable, userId, duration, timeScale);
			logger.warning("Successfully disabled service: Suspending Service <" + serviceToDisable +
					"> for userId/username <" + userId + "/" + user.getAccountName() + "> for <" + duration + "> <" + timeScale + ">");
			return true;
		} else {
			logger.warning("Could not disabled service <" + serviceToDisable + "> for " +
					"userId/username <" + userId + "/" + user.getAccountName() + "> because" +
					" there is no configuration setup in the ESAPI.properties file");
			return false;
		}
	}

	/**
	 * SMS administrator (via email to sms account) to notify of what action has occurred
	 * @param currentIntrusion current intrusion 
	 * @return true if successful
	 */
	private boolean smsAdmin(AppSensorIntrusion currentIntrusion) {
		ASUser user = currentIntrusion.getUser();
		logger.debug("Response Action: Sending SMS to Admin due to issue performed by " + user.getAccountId());
		logger.fatal("INTRUSION - Multiple intrusions observed by AppSensorUser:" + user.getAccountName() + 
				", UserID:" + user.getAccountId() + ", Action: Sending SMS to Admin with code: " + currentIntrusion.getEventCode());
		String subject = "AppSensor Issue Detection";
		String body = "User:" + user.getAccountName() + " (ID:" + user.getAccountId() + ") " +
				"/ caused event code: " + currentIntrusion.getEventCode();
		
		//truncate message for sms transmission if necessary
		int totalLength = subject.length() + body.length();		
		if (totalLength > SMS_MESSAGE_LENGTH) {
			//get leftover length we have to use for the body
			int bodyLength = SMS_MESSAGE_LENGTH - totalLength;
			body = body.substring(0, bodyLength - 1);
		}
		
		return sendEmail(MailType.SMS, subject, body);
	}
	
	/**
	 * Email administrator to notify of what action has occurred
	 * @param currentIntrusion current intrusion 
	 * @return true if successful
	 */
	private boolean emailAdmin(AppSensorIntrusion currentIntrusion) {
		ASUser user = currentIntrusion.getUser();
		logger.debug("Response Action: Sending email to Admin due to issue performed by " + user.getAccountId());
		logger.fatal("INTRUSION - Multiple intrusions observed by AppSensorUser:" + user.getAccountName() + 
				", UserID:" + user.getAccountId() + ", Action: Sending email to Admin with code: " + currentIntrusion.getEventCode());
		String subject = "AppSensor Issue Detection";
		String body = "User:" + user.getAccountName() + " (ID:" + user.getAccountId() + ") " +
				"/ caused event code: " + currentIntrusion.getEventCode();
		
		body += "\r\n" + "See http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project#tab=Detection_Points for more information.";
		
		return sendEmail(MailType.EMAIL, subject, body);
	}
	
	/**
	 * This method is used for sending email to cover both standard email 
	 * as well as SMS
	 * @param type SMS or standard email
	 * @param subject subject of email
	 * @param body body of email
	 * @return boolean indicating true if email was sent successfully, false if not
	 */
	private boolean sendEmail(MailType type, String subject, String body) {
		AppSensorSecurityConfiguration assc = (AppSensorSecurityConfiguration)AppSensorSecurityConfiguration.getInstance();
		try {
			String protocol;
			String host;
			int port;
			boolean isAuthenticationRequired = false;
			boolean isProtocolSecure = false;
			String user;
			String password;
			String from;
			List<String> toAddresses;
			
			if (MailType.EMAIL.equals(type)) {
				protocol = assc.getEmailProtocol();
				host = assc.getEmailHost();
				port = assc.getEmailPort();
				isAuthenticationRequired = assc.getEmailAuthenticationRequired();
				user = assc.getEmailAuthenticationUser();
				password = assc.getEmailAuthenticationPassword();
				from = assc.getEmailFromAccount();
				toAddresses = assc.getEmailToAccounts();
			} else if (MailType.SMS.equals(type)) {
				protocol = assc.getSMSEmailProtocol();
				host = assc.getSMSEmailHost();
				port = assc.getSMSEmailPort();
				isAuthenticationRequired = assc.getSMSEmailAuthenticationRequired();
				user = assc.getSMSEmailAuthenticationUser();
				password = assc.getSMSEmailAuthenticationPassword();
				from = assc.getSMSEmailFromAccount();
				toAddresses = assc.getSMSEmailToAccounts();
			} else {
				throw new IllegalArgumentException("Invalid MailType passed in");
			}
			
			if ("smtps".equals(protocol)) {
				isProtocolSecure = true;
				//reset protocol back to smtp to make it work properly, but 
				//do everything over https below
				protocol = "smtp";
			}
			
			Properties props = new Properties();

			if (isProtocolSecure) {
				props.put("mail.smtp.starttls.enable","true");
	        	props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
			}
			
			props.setProperty("mail.transport.protocol", protocol);
			props.setProperty("mail.host", host);
			props.setProperty("mail.port", String.valueOf(port));

			if (isAuthenticationRequired) {
				props.setProperty("mail.smtp.auth", "true");
				props.setProperty("mail.user", user);
				props.setProperty("mail.password", password);
			}
			
			Session mailSession = Session.getDefaultInstance(props, null);
			Transport transport = mailSession.getTransport(protocol);
			
			if (isAuthenticationRequired) {
				transport.connect(host, port, user, password);
			}
			
			MimeMessage message = new MimeMessage(mailSession);
			message.setFrom(new InternetAddress(from));
			for (String toAddress : toAddresses) {
				message.addRecipient(Message.RecipientType.TO, new InternetAddress(toAddress));
			}
			
			message.setSubject(subject);
			message.setText(body);
		
			transport.sendMessage(message, message.getAllRecipients());
			return true;
		} catch (Exception e) {
			String mtype;
			if (MailType.SMS.equals(type)) {
				mtype = "SMS";
			} else {
				mtype = "standard email";
			}
			logger.fatal("Could not send email (type - " + mtype + ") to admin: ", e);
			
			return false;
		}
		
	}
}
